classdef  DSPSQL_Database < handle
properties (GetAccess=private) 
  connection
  debugDSPSQL
  updatedVersion
end % PROPERTIES

methods 
function obj = DSPSQL_Database(serverName,  ...
              schemaName, authentication, debugDSPSQL_arg)
  obj.connection = actxserver('ADODB.Connection');
  obj.debugDSPSQL = debugDSPSQL_arg; 
  obj.updatedVersion = 1;
  javaaddpath(pwd);         % SET UP PATH TO SERIALIZEDATE.JAR

  connStr=sprintf('Provider=SQLOLEDB;Data Source=%s;Initial Catalog=%s;%s',...
             serverName, schemaName, authentication);
  try			% TRY TO CONNECT TO SERVER
    obj.connection.Open(connStr);           
  catch ME
    error(getReport(ME));
  end
end

%%%%%%
            
function result = DSPSQL_SendSQLQuery(obj, query)    
    errorOccurred = 0;
    cmd = actxserver('ADODB.Command');        % GET A CONNECTION
    try
      cmd.ActiveConnection = obj.connection;
    catch     % MATLAB BUG  WILL ALWAYS FAIL -- 
    end       %           IGNORE IT

    cmd.CommandText = query;
    try
    	recordSet = cmd.Execute();
    catch err
      if obj.debugDSPSQL == 1     % GOT TOO FANCY AND TRIED INVALID SQL QUERY
        fprintf('Unrecognized SQL Query: %s\n\n%s\n', query, getReport(err));
        recordSet = 0;	 
        errorOccurred = 1;
      else
        rethrow(err);             % IGNORE ERROR -- RETURN 0
      end
    end
    
    if errorOccurred
      result = 0;
    else
      if recordSet.state ~= 1
        result = 1;    % NO RECORD EXISTS                         
      else
        result = recordSet;     % RETURNED A VALID RECORD SET FROM DATA BASE
      end
    end
end

% function value = DSPSQL_GetConnection(obj)
%   value = obj.connection; % RETURNS THE CONNECTION OBJECT FOR THIS INSTANCE
% end

%%%%%%%%%%%%%%%%%%%%%%%%%


function tableExists = DSPSQL_TableExists(obj, tableName)
  recordSet  = obj.connection.OpenSchema('adSchemaTables').GetRows;
  tableNames = recordSet (3,ismember(recordSet (4,:),'TABLE'));

  if ~ismember(tableName, tableNames)
    tableExists = 0;
  else
    tableExists = 1;
  end
end

%%%%%%

function DSPSQL_AddNewTable(obj, tableName, varargin)
  if obj.updatedVersion == 1
    SQLQuery = sprintf('CREATE TABLE dbo.%s (Record_ID float',  ...
                                             tableName);
    % Last element is always a BLOB
    for temp = 1 : (nargin-4)
      SQLQuery = sprintf('%s,%s float', SQLQuery, varargin{temp});
    end
    SQLQuery = sprintf('%s,%s varbinary(max) )', SQLQuery, varargin{nargin - 3});
    % SQLQuery
    obj.DSPSQL_SendSQLQuery(SQLQuery);
  else
    SQLQuery = sprintf('CREATE TABLE dbo.%s (Record_ID int,%s varbinary(max)',  ...
                                             tableName,varargin{1});
    for temp = 2 : (nargin-2)
      SQLQuery = sprintf('%s,%s varbinary(max)', SQLQuery, varargin{temp});
    end
    SQLQuery = sprintf('%s)',  SQLQuery);
    obj.DSPSQL_SendSQLQuery(SQLQuery);
  end
end

%%%%%%

function newRecordNumber = DSPSQL_NewRecord(obj,  tableName, varargin)

    cmd = actxserver('ADODB.Command');   % Make this part of the constructor
    try
      cmd.ActiveConnection = obj.connection;
    catch % IGNORE MATLAB BUG
    end

    % FIND Record_ID OF LAST RECORD IN TABLE
    SQL_Query = sprintf('SELECT MAX(Record_ID) FROM dbo.%s', tableName);
    cmd.CommandText = SQL_Query;
    lastID = cmd.Execute().GetRows{1};
    if isnan(lastID)                  % 
      lastID = 0;
    end
    newRecordNumber = lastID + 1;

   if obj.updatedVersion == 0
    SQL_Query = sprintf('INSERT INTO dbo.%s VALUES(%d,?',     ...
              tableName, newRecordNumber);     
          % ???????,  %NEEDS DATE INSERTED

    % MAGIC BACKDOOR WHEN TELLING SERVER ABOUT BLOBS
    BLOBdata = SerializeData.WriteToBytes(varargin{1});
    cmd.Parameters.Append(cmd.CreateParameter('', 205, 1, -1, BLOBdata'));

    for temp = 2:(nargin-2)
      SQL_Query = sprintf('%s,?', SQL_Query);
      BLOBdata = SerializeData.WriteToBytes(varargin{temp});
      cmd.Parameters.Append(cmd.CreateParameter('', 205, 1, -1, BLOBdata'));
    end
   else
    SQL_Query = sprintf('INSERT INTO dbo.%s VALUES(%d',     ...
            tableName, newRecordNumber);     
        % ???????,  %NEEDS DATE INSERTED

  % Everything is not a BLOB except last field
     for temp = 1:(nargin-3)
        SQL_Query = sprintf('%s,%f', SQL_Query, varargin{temp});
     end
     % MAGIC BACKDOOR WHEN TELLING SERVER ABOUT BLOBS
     SQL_Query = sprintf('%s,?', SQL_Query);
     BLOBdata = SerializeData.WriteToBytes(varargin{nargin-2});
     
     cmd.Parameters.Append(cmd.CreateParameter('', 205, 1, -1, BLOBdata'));
   end
  % SQL_Query
  cmd.CommandText  = sprintf('%s)', SQL_Query);
  cmd.Execute();
end

%%%%%%%%%%%%%%%%%%%%%%%%%

function recording = DSPSQL_RetrieveRecord(obj, dataTableName, varargin)
  if obj.updatedVersion == 1
    SQL_Query = sprintf('SELECT * FROM dbo.%s WHERE', dataTableName);
    
    for count = 1 : 2: nargin - 4
      SQL_Query = sprintf('%s %s=%d AND', ...
                  SQL_Query, varargin{count}, varargin{count + 1});
    end
    SQL_Query = sprintf('%s %s=%d', ...
                  SQL_Query, varargin{nargin - 3}, varargin{nargin - 2});
    % SQL_Query
    recordSet = DSPSQL_SendSQLQuery(obj, SQL_Query);
    recording = DSP_Record(recordSet);
  else
    recording = 0;   
  end
end

%%%%%%


function numCommands = DSPSQL_GetNumberCommands(obj, dataTableName)
  SQL_Query = sprintf('SELECT COUNT(DISTINCT %s) AS SQL_QueryResult FROM dbo.%s', ...
                                    'commandNumber', dataTableName);
  recordSet = DSPSQL_SendSQLQuery(obj, SQL_Query);
  recording = DSP_Record(recordSet);
  numCommands = recording.DSP_GetFieldValue('SQL_QueryResult');
end

function numExamples = DSPSQL_GetNumberExamples(obj, dataTableName, commandNumber)
  SQL_Query = sprintf('SELECT MAX(exampleNumber) AS SQL_QueryResult FROM ');
  SQL_Query = sprintf('%s dbo.%s WHERE commandNumber=%d', ...
                          SQL_Query, dataTableName, commandNumber);
  recordSet = DSPSQL_SendSQLQuery(obj, SQL_Query);
  numExamples = DSP_Record(recordSet).DSP_GetFieldValue('SQL_QueryResult');
end

%%%%%%

function result = DSPSQL_MaxInField(obj, dataTableName, fieldName)
  SQL_Query = sprintf('SELECT COUNT(DISTINCT %s) AS SQL_QueryResult FROM dbo.%s', ...
                                    fieldName, dataTableName);
  recordSet = DSPSQL_SendSQLQuery(obj, SQL_Query);
  recording = DSP_Record(recordSet);
  result = recording.DSP_GetFieldValue('SQL_QueryResult');
end


  
  

end % METHODS
end % CLASS
