function DSPSQL_main( )        % DSPSQL_main.m
  debugDSPSQL = 1;            % ACTIVATE DSPSQL DEBUG MODE
  % CONNECT TO DATABASE SERVER USING DSP_SQL_DATABASE CLASS
  dbObject = DSPSQL_Database('localhost\SQLEXPRESS', 'CCArticle',  ...
              'Trusted_Connection=Yes', debugDSPSQL); % NO SPACES AROUND =

  % STORE OR RETRIEVE THE DSP SIGNALS
  tic
  MainDataTable(dbObject, 'SimulatedSnores', 'CHIRP');
  toc
% MainDataTable(dbObject, 'Mind2Sail', 'EEG');
% MainDataTable(dbObject, 'RecordedPhrases', 'VOICE'); 
  % ETC.

  % PERFORM COMPARATIVE ANALYSIS VIA FUNCTION POINTERS
  tic
  PerformAnalysis(dbObject, 'SimulatedSnores', 'Correlation', @Correlation);
  toc
% PerformAnalysis(dbObject, 'SimulatedSnores', 'PowerFreqBands', ...
%               @PowerFreqBands);
% PerformAnalysis(dbObject, 'Mind2Sail', 'Correlation', @Correlation);   
% ETC.

  % GENERATE COMPARATIVE REPORTS
  tic
  ComparisonReport(dbObject, 'SimulatedSnores',      ...         
              'Correlation', @Correlation); %, 'PowerFreqBands', @PowerFreqBands);
  toc
% ComparisonReport(dbObject, 'Mind2Sail',    ...    
%              'Correlation', @Correlation); %, 'PowerFreqBands', @PowerFreqBands);
% ETC.
end

%%%%%%

 function  MainDataTable(obj, dataTable, recordingDevice) % MainDataTable.m
  DSPSQL_SendSQLQuery(obj, sprintf('DROP TABLE dbo.%s', dataTable));
  % RETURN IF THE DATA TABLE ALREADY EXISTS
  if  DSPSQL_TableExists(obj, dataTable)
    fprintf('Data table <%s> exists\n', dataTable);
    return;
  end

  % OTHERWISE ADD A NEW DATA TABLE TO THE SERVER DATA BASE
  %  DEFAULT FIELDS Record_ID AND Capture_Date ' ADDED AUTOMOTACIALLY
  DSPSQL_AddNewTable(obj, dataTable , 'CommandNumber',...
              'ExampleNumber', 'SampleRateHz', 'RecordedData', 'BLOB');

  % AND FILL THE TABLE WITH NEW DSP INFORMATION FROM THE RECORDING DEVICE                                                                         
  for numCommand = 1 : 4    % DEFAULT TO AN INITIAL SET OF 4 COMMANDS
    for numExample = 1 : 60     % WITH 6 EXAMPLES OF EACH COMMAND
    recording = GetDeviceRecording(recordingDevice, ...
                                 numCommand, numExample);
    DSPSQL_NewRecord(obj, dataTable , numCommand, numExample, ...
                                recording.sampleRateHz, recording.data);
    end
  end
 end
 
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
 
 function PerformAnalysis(obj, dataTableName, analysisName,  functionHandle)                                                                                           
  analysisTableName = sprintf('%s_%s', dataTableName, analysisName);
  DSPSQL_SendSQLQuery(obj, sprintf('DROP TABLE dbo.%s', analysisTableName));
  % BUILD RESULT TABLE IF NEEDED
  if DSPSQL_TableExists(obj, analysisTableName) ~= 1
    functionHandle(obj, 'BuildTable', analysisTableName);        
  else
    fprintf('Results table <%s> exists\n', analysisTableName);
    return;
  end

  % PERFORM ANALYSIS ON ALL THE EXAMPLES FOR EACH COMMAND 
  numCommands = DSPSQL_GetNumberCommands(obj, dataTableName); 
  showProgress = waitbar(0, 'Calculating Correlation results');
  for command = 1 : numCommands
    waitbar(command / numCommands);   % PROGRESS BAR IS USEFUL
    numExamples = DSPSQL_GetNumberExamples(obj, dataTableName, command);   
    for example = 1 : numExamples
      functionHandle(obj, dataTableName, analysisTableName , command, example);
    end
  end
  close(showProgress);
 end
 
 %%%%%%
 
function ComparisonReport(obj, dataTableName,  varargin)
  %  FOR EVERY ANALYSIS APPROACH PREPARE AN ROC REPORT 
  %             A LARGE AREA, AZ, UNDER ROC CURVE SHOWS COMMAND DESCRIMINATION
  figure;
  for analysisParameters = 1 : 2 : nargin - 2
    analysisTableName = sprintf('%s_%s', dataTableName, varargin{analysisParameters});
    functionHandle =  varargin{analysisParameters + 1};
    
    % PERFORM ROC ANALYSIS FOR EACH COMMAND 
    numCommands = DSPSQL_GetNumberCommands(obj, dataTableName);
    for command = 1 : numCommands
      [posTable, negTable] = functionHandle(obj, dataTableName,  ... 
                          analysisTableName , command,  'ROC_TABLE');
      fprintf('ROC AZ FOR COMMAND %d IN ANALYSIS %s IS %f, LARGER BETTER\n', ...
            command, varargin{analysisParameters}, ROC_Analysis(posTable, negTable, command));
    end
  end
  x = 0:.1:1;
  y = 0:.1:1;
  
  plot( [0, 1], [0, 1], '-.b', 'linewidth', 2);
  xlabel('\fontsize{20}FALSE POSITIVE RATE');
  ylabel('\fontsize{20}TRUE POSITIVE RATE');
  legend('\fontsize{14}PERSON1', '\fontsize{14}PERSON 2', '\fontsize{14}PERSON 3', '\fontsize{14}PERSON 4', '\fontsize{14}NO DECISION', 'LOCATION', 'BEST');
end


%%%%%%%%%%%%%%%%%%%%%%%%%


function recording = GetDeviceRecording(recordingDevice, ...
                numCommand, numExample)
  fprintf('Inputting <command %d> example %d from device %s\n', ...
                numCommand, numExample,recordingDevice);

  if strcmp(recordingDevice, 'CHIRP')               % WIDER CHIRPS WORK THE BEST
    recording.sampleRateHz = 44000;
    deltaT = 1 / recording.sampleRateHz;      % SAMPLE INTERVAL

    % SET CHIRP CHARACTERISTICS AND GENERATE RECORDING
    centreFrequency = 1000 + numCommand * 1000 + 1000 *(rand(1) - 0.5);
    startFrequency = centreFrequency - 1500;
    
    time = 0 : deltaT : (4096 - 1) * deltaT;
    approxHalfTime =  2048 * deltaT;
    recording.data = chirp(time, startFrequency,  ...
                 approxHalfTime,  centreFrequency); 

  elseif strcmp(recordingDevice, 'VOICE')        % GRAB AUDIO FROM A MIKE
    recording.sampleRateHz = 44000 / 8;   % SET AUDIO INFORMATION
    sampleBitSize = 24; 
    numChannels = 2;
    recordingLength = 2;                                 % RECORD 2 SECONDS

    % USE MATLAB INBUILT FUNCTIONALITY
    recordingObject = audiorecorder(recording.sampleRateHz, ...
                  sampleBitSize, numChannels);
    recordingObject.recordblocking(recordingLength);
    recording.data = recordingObject.getaudiodata( );   

  %  IN PRACTICE, WE USED A NEUROSKY API TO CAPTURE MINDWAVE EEG DATA 
  %       AND DIRECTLY STORE INTO THE DATABASE USING A C_SQL INTERFACE
  %       A SIMILAR APPROACH WAS USED FOR THE HUM DATA 
  %  THE FOLLOWING DEMONSTRATES AN APPROACH TO DYNAMICALLY
  %               CAPTURE NEW EEG DATA 
  elseif strcmp(recordingDevice, 'EEG')
    % USE MESSAGES TO COMMUNICATE WITH EXTERNAL CAPTURE PROGRAM
    pause();
    while IsReady('EEGCapture\Message') ~= 1;   
      pause(5);      %   USE PAUSES IN PROTOTYPE TO AVOID DATA RACES 
    end  

    fileID = fopen('EEGCapture\Message', 'rt');  %   REQUEST SAMPLE
    fprintf(fileID, '0, %d, %', numCommand, numExample);
    fclose(fileID);

    pause(20);
    while IsReady('EEG\EEGCapture\Message')  ~= 1;
      pause(20);      
    end     

    fileID = fopen('EEGCapture\Message', 'rt');
    fgets(FID);   % Ignore first line
    recording.sampleRateHz = fscanf('%f', fgets(fileID));
    numPtsRecorded = fscanf('%d', fgets(fileID));

    recording.data = zeros(1, numPtsRecorded);
    for count = 1 : numPtsRecorded
      recording.data(count) = fscanf('%f', fgets(fileID));
    end
  end
end


%%%%%%%%%%%%%%%%%%%%%%%%%


function [posTable, negTable] = Correlation(obj, varargin) % CORRELATION ANALYSIS 
  if  strcmp('BuildTable', varargin{1})  % REQUEST TO BUILD A TABLE?
    DSPSQL_AddNewTable(obj, varargin{2},   ...
                    'Command1', 'Example1', 'Command2', 'Example2', ...
                      'SampleRateHz', 'Correlation_3dBWidth', 'Correlation_6dbWidth', ...
                              'CorrelationData', 'BLOB');
    posTable = 0;
    return;
  end

  dataTableName = varargin{1};            
  analysisTableName = varargin{2};
  command1 = varargin{3}; 
  example1 = varargin{4};

  if strcmp(varargin{4}, 'ROC_TABLE')  % RETURN ANALYSIS RESULTS FOR ROC
    [posTable, negTable] = GetCorrelationResults(obj, dataTableName ,  ...
                                      analysisTableName,  command1); 
    return
  end

  % GRAB THE REQUIRED EXAMPLE FROM FIRST COMMAND  
  firstDataSet = DSPSQL_RetrieveRecord(obj, dataTableName, ...
                  'CommandNumber', command1, 'ExampleNumber', example1);
  sampleRateHz = firstDataSet.DSP_GetFieldValue('SampleRateHz');

  % PERFORM CORRELATION WITH ALL EXAMPLES OF ALL COMMANDS  
  numberOtherCommands = DSPSQL_GetNumberCommands(obj, dataTableName); 
  for command2 = command1 : numberOtherCommands
    numExamples2 = DSPSQL_GetNumberExamples(obj, dataTableName, command2);
    for example2 = 1 : numExamples2
      secondDataSet = DSPSQL_RetrieveRecord(obj, dataTableName, ...
               'CommandNumber', command2, 'ExampleNumber', example2);
      result = xcorr(firstDataSet.DSP_GetFieldValue('RecordedData'), ... 
                                secondDataSet.DSP_GetFieldValue('RecordedData'));                         
      correlation_3dBWidth = Calculate3dBWidth(result);
      correlation_6dBWidth = Calculate6dBWidth(result);
     
      % STORE THE ANALYSIS RESULTS IN THE DATA BASE
      DSPSQL_NewRecord(obj, analysisTableName, command1, example1, ...
                command2, example2, sampleRateHz, ...
                    correlation_3dBWidth, correlation_6dBWidth, result);
    end
  end
end


%%%%%%%%%%%%%%%%%%%%%%%%%

 
function  [posTable, negTable] = GetCorrelationResults(obj, dataTableName, ...
                                      analysisTableName,  command1)
  % HANDLE AUTO-CORRELATION RESULTS
  numExamples1 = DSPSQL_GetNumberExamples(obj, dataTableName, command1);
  count = 1;
  for example1 = 1 :  numExamples1
    for example2 = example1 : numExamples1
      autoCorrRecord = DSPSQL_RetrieveRecord(obj, analysisTableName,  ...
                      'Command1', command1, 'Example1', example1, ...
                        'Command2', command1, 'Example2', example2);
      posTable(count, 1) = autoCorrRecord.DSP_GetFieldValue('Correlation_3dBWidth') ;
      posTable(count, 2) = autoCorrRecord.DSP_GetFieldValue('Correlation_6dBWidth') ;
      count = count + 1;      % SLOW APPROACH FOR LARGE RECORD SETS
    end
  end
   fprintf('\n');
  % HANDLE CROSS-CORRELATION RESULTS                                                                                               
  count = 1;      
  numCommands = DSPSQL_GetNumberCommands(obj, dataTableName);  
  for command2 = 1 : numCommands
    if command1 ~= command2
      numExamples2 = DSPSQL_GetNumberExamples(obj, dataTableName, command2);
      for example1 = 1 : numExamples1
        for example2 = 1 : numExamples2
          tempCommand1 = command1;
          tempCommand2 = command2;
          tempExample1 = example1;
          tempExample2 = example2;
          
          if (command2 < command1)
             tempCommand1 = command2;
            tempCommand2 = command1;
            tempExample1 = example2;
            tempExample2 = example1;
          end
          crossCorrRecord = DSPSQL_RetrieveRecord(obj, analysisTableName,  ...
                              'Command1', tempCommand1, 'Example1', tempExample1, ...
                                  'Command2', tempCommand2, 'Example2', tempExample2);
          negTable(count, 1) = crossCorrRecord.DSP_GetFieldValue('Correlation_3dBWidth');
          negTable(count, 2) = crossCorrRecord.DSP_GetFieldValue('Correlation_6dBWidth');
          count = count + 1;   % SLOW APPROACH FOR LARGE RECORD SETS
        end 
      end
    end
  end
end
 
 
%%%%%%%%%%%%%%%%%%%%%%%%%

function result = Calculate3dBWidth(correlationTimeData)
  data = abs(correlationTimeData);
  numPts = length(data);

  [maxValue, maxPosn] = max(data);

  if maxValue < 200
    result = 200;
    return;
  end
  
  maxValueBy2 = maxValue / 2;
  for count = 1 : numPts
    if data(count) > maxValueBy2
      data(count) = maxValueBy2;
    end
  end
  
  [temp, halfPt] = max(data);
  result = abs(maxPosn - halfPt);
end

%%%%%%

function result = Calculate6dBWidth(correlationTimeData)
    data = abs(correlationTimeData);
  numPts = length(data);
  [maxValue, maxPosn] = max(data);

  if maxValue < 200
    result = 200;
    return;
  end
  
  maxValueBy4 = maxValue / 4;
  for count = 1 : numPts
    if data(count) > maxValueBy4
      data(count) = maxValueBy4;
    end
  end
  
  [temp, quarterPt] = max(data);
  result = abs(maxPosn - quarterPt);
end

%%%%%%

function result = ROC_Analysis(posTable, negTable, command)
  size(posTable);
  size(negTable);
  pos = posTable(:, 1);
  neg = negTable(:, 1);

%   figure;
%   plot(pos, '-k'); hold on;
%   plot(neg, '-b'); hold on;
  
  count = 1;
  for numPos = 1: length(pos)
    scores(count) = posTable(numPos);
    labels(count) = 1;
    count = count + 1;
  end
   for numNeg = 1: length(neg)
    scores(count) = negTable(numNeg);
    labels(count) = 0;
    count = count + 1;
  end
  
  ROC( labels, scores, command);
  result = 1;
end
   
%%%%%%

function [Tps, Fps] = ROC(labels, scores, command)
if 1
% Sort Labels and Scores by Scores
  sl = [scores; labels];
  [d1 d2] = sort(sl(1,:));

  sorted_sl = sl(:,d2);
  s_scores = sorted_sl(1,:);
  s_labels = round(sorted_sl(2,:));

  % Constants
  counts = histc(s_labels, unique(s_labels));

  Tps = zeros(1, size(s_labels,2) + 1);
  Fps = zeros(1,  size(s_labels,2) + 1);

  negCount = counts(1);
  posCount = counts(2);

  % Shift threshold to find the ROC
  for thresIdx = 1:(size(s_scores,2)+1)

    % for each Threshold Index
    tpCount = 0;
    fpCount = 0;

    for i = [1:size(s_scores,2)]
      if (i >= thresIdx)           % We think it is positive
        if (s_labels(i) == 1)   % Right!
          tpCount = tpCount + 1;
        else                    % Wrong!
          fpCount = fpCount + 1;
        end
      end
    end

    Tps(thresIdx) = tpCount/posCount;
    Fps(thresIdx) = fpCount/negCount;
  end

  % Draw the Curve

  % Sort [Tps;Fps]
  x = Tps;
  y = Fps;

  % Interplotion to draw spline line
  count = 100;
  dist = (x(1) - x(size(x,2)))/100;
  xx = [x(1):-dist:x(size(x,2))];

  % In order to get the interpolations, we remove all the unique numbers
  [d1 d2] = unique(x);
  uni_x = x(1,d2);
  uni_y = y(1,d2);
  yy = spline(uni_x,uni_y,xx);

  % No value should exceed 1
  yy = min(yy, 1);

  % plot(x,y,'x',xx,yy);  hold on; 
  switch command
    case 1 
      plot(xx,yy, '-k', 'linewidth', 2);  hold on;
    case 2 
      plot(xx,yy, '-r', 'linewidth', 2);  hold on; 
    case 3 
      plot(xx,yy, '--r', 'linewidth', 2);  hold on; 
    case 4 
      plot(xx,yy, '--k', 'linewidth', 2);  hold on;
  end

 
else
 % function [tp, fp] = ROC(t,y)

%
% ROC - generate a receiver operating characteristic curve
%
%    [TP,FP] = ROC(T,Y) gives the true-positive rate (TP) and false positive
%    rate (FP), where Y is a column vector giving the score assigned to each
%    pattern and T indicates the true class (a value above zero represents
%    the positive class and anything else represents the negative class).  To
%    plot the ROC curve,
%
%       PLOT(FP,TP);
%       XLABEL('FALSE POSITIVE RATE');
%       YLABEL('TRUE POSITIVE RATE');
%       TITLE('RECEIVER OPERATING CHARACTERISTIC (ROC)');
%
%    See [1] for further information.
%
%    [1] Fawcett, T., "ROC graphs : Notes and practical
%        considerations for researchers", Technical report, HP
%        Laboratories, MS 1143, 1501 Page Mill Road, Palo Alto
%        CA 94304, USA, April 2004.
%
%    See also : ROCCH, AUROC

%
% File        : roc.m
%
% Date        : Friday 9th June 2005
%
% Author      : Dr Gavin C. Cawley
%
% Description : Generate an ROC curve for a two-class classifier.
%
% References  : [1] Fawcett, T., "ROC graphs : Notes and practical
%                   considerations for researchers", Technical report, HP
%                   Laboratories, MS 1143, 1501 Page Mill Road, Palo Alto
%                   CA 94304, USA, April 2004.
%
% History     : 10/11/2004 - v1.00
%               09/06/2005 - v1.10 - minor recoding
%               05/09/2008 - v2.00 - re-write using algorithm from [1]
%
% Copyright   : (c) G. C. Cawley, September 2008.
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program; if not, write to the Free Software
%    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
%

ntp = size(y,1);

% sort by classeifier output

[y,idx] = sort(y, 'descend');
t       = t(idx) > 0;

% generate ROC

P     = sum(t);
N     = ntp - P;
fp    = zeros(ntp+2,1);
tp    = zeros(ntp+2,1);
FP    = 0;
TP    = 0;
n     = 1;
yprev = -realmax;

for i=1:ntp

   if y(i) ~= yprev

      tp(n) = TP/P;
      fp(n) = FP/N; 
      yprev = y(i);
      n     = n + 1;

   end

   if t(i) == 1

      TP = TP + 1;

   else

      FP = FP + 1;

   end

end

tp(n) = 1;
fp(n) = 1;
fp    = fp(1:n);
tp    = tp(1:n);

figure;
plot(fp, tp);

% bye bye...
end
end
 
 
 
 
 
 
 
 